@@ -1,10 +1,12 @@
 #!/usr/bin/env python
 
 """
-Copyright (c) 2006-2018 sqlmap developers (http://sqlmap.org/)
+Copyright (c) 2006-2019 sqlmap developers (http://sqlmap.org/)
 See the file 'LICENSE' for copying permission
 """
 
+from __future__ import print_function
+
 try:
     from crypt import crypt
 except:  # removed ImportError because of https://github.com/sqlmapproject/sqlmap/issues/3171
@@ -14,12 +16,12 @@ _multiprocessing = None
 try:
     import multiprocessing
 
-    # problems on FreeBSD (Reference: http://www.eggheadcafe.com/microsoft/Python/35880259/multiprocessing-on-freebsd.aspx)
+    # problems on FreeBSD (Reference: https://web.archive.org/web/20110710041353/http://www.eggheadcafe.com/microsoft/Python/35880259/multiprocessing-on-freebsd.aspx)
     _ = multiprocessing.Queue()
 
     # problems with ctypes (Reference: https://github.com/sqlmapproject/sqlmap/issues/2952)
     _ = multiprocessing.Value('i')
-except (ImportError, OSError):
+except (ImportError, OSError, AttributeError):
     pass
 else:
     try:
@@ -31,6 +33,7 @@ else:
 import base64
 import binascii
 import gc
+import hashlib
 import os
 import re
 import tempfile
@@ -43,7 +46,6 @@ from hashlib import sha224
 from hashlib import sha256
 from hashlib import sha384
 from hashlib import sha512
-from Queue import Queue
 
 from lib.core.common import Backend
 from lib.core.common import checkFile
@@ -52,20 +54,26 @@ from lib.core.common import dataToStdout
 from lib.core.common import getFileItems
 from lib.core.common import getPublicTypeMembers
 from lib.core.common import getSafeExString
-from lib.core.common import getUnicode
 from lib.core.common import hashDBRetrieve
 from lib.core.common import hashDBWrite
+from lib.core.common import isZipFile
 from lib.core.common import normalizeUnicode
+from lib.core.common import openFile
 from lib.core.common import paths
 from lib.core.common import readInput
 from lib.core.common import singleTimeLogMessage
 from lib.core.common import singleTimeWarnMessage
-from lib.core.convert import hexdecode
-from lib.core.convert import hexencode
-from lib.core.convert import utf8encode
+from lib.core.compat import xrange
+from lib.core.convert import decodeBase64
+from lib.core.convert import decodeHex
+from lib.core.convert import encodeHex
+from lib.core.convert import getBytes
+from lib.core.convert import getText
+from lib.core.convert import getUnicode
 from lib.core.data import conf
 from lib.core.data import kb
 from lib.core.data import logger
+from lib.core.datatype import OrderedSet
 from lib.core.enums import DBMS
 from lib.core.enums import HASH
 from lib.core.enums import MKSTEMP_PREFIX
@@ -75,28 +83,34 @@ from lib.core.settings import COMMON_PASSWORD_SUFFIXES
 from lib.core.settings import COMMON_USER_COLUMNS
 from lib.core.settings import DEV_EMAIL_ADDRESS
 from lib.core.settings import DUMMY_USER_PREFIX
+from lib.core.settings import HASH_BINARY_COLUMNS_REGEX
+from lib.core.settings import HASH_EMPTY_PASSWORD_MARKER
 from lib.core.settings import HASH_MOD_ITEM_DISPLAY
 from lib.core.settings import HASH_RECOGNITION_QUIT_THRESHOLD
+from lib.core.settings import INVALID_UNICODE_CHAR_FORMAT
 from lib.core.settings import IS_WIN
 from lib.core.settings import ITOA64
 from lib.core.settings import NULL
-from lib.core.settings import UNICODE_ENCODING
 from lib.core.settings import ROTATING_CHARS
+from lib.core.settings import UNICODE_ENCODING
 from lib.core.wordlist import Wordlist
+from thirdparty import six
 from thirdparty.colorama.initialise import init as coloramainit
-from thirdparty.oset.pyoset import oset
-from thirdparty.pydes.pyDes import des
 from thirdparty.pydes.pyDes import CBC
+from thirdparty.pydes.pyDes import des
+from thirdparty.six.moves import queue as _queue
 
 def mysql_passwd(password, uppercase=True):
     """
     Reference(s):
-        http://csl.sublevel3.org/mysql-password-function/
+        https://web.archive.org/web/20120215205312/http://csl.sublevel3.org/mysql-password-function/
 
     >>> mysql_passwd(password='testpass', uppercase=True)
     '*00E247AC5F9AF26AE0194B41E1E769DEE1429A29'
     """
 
+    password = getBytes(password)
+
     retVal = "*%s" % sha1(sha1(password).digest()).hexdigest()
 
     return retVal.upper() if uppercase else retVal.lower()
@@ -104,8 +118,8 @@ def mysql_passwd(password, uppercase=True):
 def mysql_old_passwd(password, uppercase=True):  # prior to version '4.1'
     """
     Reference(s):
-        http://www.sfr-fresh.com/unix/privat/tpop3d-1.5.5.tar.gz:a/tpop3d-1.5.5/password.c
-        http://voidnetwork.org/5ynL0rd/darkc0de/python_script/darkMySQLi.html
+        https://web.archive.org/web/20091205000600/http://www.sfr-fresh.com/unix/privat/tpop3d-1.5.5.tar.gz:a/tpop3d-1.5.5/password.c
+        https://github.com/pwnieexpress/pwn_plug_sources/blob/master/src/darkmysqli/DarkMySQLi.py
 
     >>> mysql_old_passwd(password='testpass', uppercase=True)
     '7DCDA0D57290B453'
@@ -135,17 +149,31 @@ def postgres_passwd(password, username, uppercase=False):
     'md599e5ea7a6f7c3269995cba3927fd0093'
     """
 
-    if isinstance(username, unicode):
-        username = unicode.encode(username, UNICODE_ENCODING)
-
-    if isinstance(password, unicode):
-        password = unicode.encode(password, UNICODE_ENCODING)
+    username = getBytes(username)
+    password = getBytes(password)
 
     retVal = "md5%s" % md5(password + username).hexdigest()
 
     return retVal.upper() if uppercase else retVal.lower()
 
-def mssql_passwd(password, salt, uppercase=False):
+def mssql_new_passwd(password, salt, uppercase=False):  # since version '2012'
+    """
+    Reference(s):
+        http://hashcat.net/forum/thread-1474.html
+        https://sqlity.net/en/2460/sql-password-hash/
+
+    >>> mssql_new_passwd(password='testpass', salt='4086ceb6', uppercase=False)
+    '0x02004086ceb6eb051cdbc5bdae68ffc66c918d4977e592f6bdfc2b444a7214f71fa31c35902c5b7ae773ed5f4c50676d329120ace32ee6bc81c24f70711eb0fc6400e85ebf25'
+    """
+
+    binsalt = decodeHex(salt)
+    unistr = b"".join((_.encode(UNICODE_ENCODING) + b"\0") if ord(_) < 256 else _.encode(UNICODE_ENCODING) for _ in password)
+
+    retVal = "0200%s%s" % (salt, sha512(unistr + binsalt).hexdigest())
+
+    return "0x%s" % (retVal.upper() if uppercase else retVal.lower())
+
+def mssql_passwd(password, salt, uppercase=False):  # versions '2005' and '2008'
     """
     Reference(s):
         http://www.leidecker.info/projects/phrasendrescher/mssql.c
@@ -155,14 +183,14 @@ def mssql_passwd(password, salt, uppercase=False):
     '0x01004086ceb60c90646a8ab9889fe3ed8e5c150b5460ece8425a'
     """
 
-    binsalt = hexdecode(salt)
-    unistr = "".join(("%s\0" if ord(_) < 256 else "%s") % utf8encode(_) for _ in password)
+    binsalt = decodeHex(salt)
+    unistr = b"".join((_.encode(UNICODE_ENCODING) + b"\0") if ord(_) < 256 else _.encode(UNICODE_ENCODING) for _ in password)
 
     retVal = "0100%s%s" % (salt, sha1(unistr + binsalt).hexdigest())
 
     return "0x%s" % (retVal.upper() if uppercase else retVal.lower())
 
-def mssql_old_passwd(password, salt, uppercase=True):  # prior to version '2005'
+def mssql_old_passwd(password, salt, uppercase=True):  # version '2000' and before
     """
     Reference(s):
         www.exploit-db.com/download_pdf/15537/
@@ -173,29 +201,13 @@ def mssql_old_passwd(password, salt, uppercase=True):  # prior to version '2005'
     '0x01004086CEB60C90646A8AB9889FE3ED8E5C150B5460ECE8425AC7BB7255C0C81D79AA5D0E93D4BB077FB9A51DA0'
     """
 
-    binsalt = hexdecode(salt)
-    unistr = "".join(("%s\0" if ord(_) < 256 else "%s") % utf8encode(_) for _ in password)
+    binsalt = decodeHex(salt)
+    unistr = b"".join((_.encode(UNICODE_ENCODING) + b"\0") if ord(_) < 256 else _.encode(UNICODE_ENCODING) for _ in password)
 
     retVal = "0100%s%s%s" % (salt, sha1(unistr + binsalt).hexdigest(), sha1(unistr.upper() + binsalt).hexdigest())
 
     return "0x%s" % (retVal.upper() if uppercase else retVal.lower())
 
-def mssql_new_passwd(password, salt, uppercase=False):
-    """
-    Reference(s):
-        http://hashcat.net/forum/thread-1474.html
-
-    >>> mssql_new_passwd(password='testpass', salt='4086ceb6', uppercase=False)
-    '0x02004086ceb6eb051cdbc5bdae68ffc66c918d4977e592f6bdfc2b444a7214f71fa31c35902c5b7ae773ed5f4c50676d329120ace32ee6bc81c24f70711eb0fc6400e85ebf25'
-    """
-
-    binsalt = hexdecode(salt)
-    unistr = "".join(("%s\0" if ord(_) < 256 else "%s") % utf8encode(_) for _ in password)
-
-    retVal = "0200%s%s" % (salt, sha512(unistr + binsalt).hexdigest())
-
-    return "0x%s" % (retVal.upper() if uppercase else retVal.lower())
-
 def oracle_passwd(password, salt, uppercase=True):
     """
     Reference(s):
@@ -207,9 +219,10 @@ def oracle_passwd(password, salt, uppercase=True):
     'S:2BFCFDF5895014EE9BB2B9BA067B01E0389BB5711B7B5F82B7235E9E182C'
     """
 
-    binsalt = hexdecode(salt)
+    binsalt = decodeHex(salt)
+    password = getBytes(password)
 
-    retVal = "s:%s%s" % (sha1(utf8encode(password) + binsalt).hexdigest(), salt)
+    retVal = "s:%s%s" % (sha1(password + binsalt).hexdigest(), salt)
 
     return retVal.upper() if uppercase else retVal.lower()
 
@@ -224,20 +237,26 @@ def oracle_old_passwd(password, username, uppercase=True):  # prior to version '
 
     IV, pad = "\0" * 8, "\0"
 
-    if isinstance(username, unicode):
-        username = unicode.encode(username, UNICODE_ENCODING)
+    unistr = b"".join((b"\0" + _.encode(UNICODE_ENCODING)) if ord(_) < 256 else _.encode(UNICODE_ENCODING) for _ in (username + password).upper())
 
-    if isinstance(password, unicode):
-        password = unicode.encode(password, UNICODE_ENCODING)
-
-    unistr = "".join("\0%s" % c for c in (username + password).upper())
-
-    cipher = des(hexdecode("0123456789ABCDEF"), CBC, IV, pad)
+    cipher = des(decodeHex("0123456789ABCDEF"), CBC, IV, pad)
     encrypted = cipher.encrypt(unistr)
     cipher = des(encrypted[-8:], CBC, IV, pad)
     encrypted = cipher.encrypt(unistr)
 
-    retVal = hexencode(encrypted[-8:])
+    retVal = encodeHex(encrypted[-8:], binary=False)
+
+    return retVal.upper() if uppercase else retVal.lower()
+
+def md4_generic_passwd(password, uppercase=False):
+    """
+    >>> md4_generic_passwd(password='testpass', uppercase=False)
+    '5b4d300688f19c8fd65b8d6ccf98e0ae'
+    """
+
+    password = getBytes(password)
+
+    retVal = hashlib.new("md4", password).hexdigest()
 
     return retVal.upper() if uppercase else retVal.lower()
 
@@ -247,6 +266,8 @@ def md5_generic_passwd(password, uppercase=False):
     '179ad45c6ce2cb97cf1029e212046e81'
     """
 
+    password = getBytes(password)
+
     retVal = md5(password).hexdigest()
 
     return retVal.upper() if uppercase else retVal.lower()
@@ -257,6 +278,8 @@ def sha1_generic_passwd(password, uppercase=False):
     '206c80413b9a96c1312cc346b7d2517b84463edd'
     """
 
+    password = getBytes(password)
+
     retVal = sha1(password).hexdigest()
 
     return retVal.upper() if uppercase else retVal.lower()
@@ -267,7 +290,9 @@ def apache_sha1_passwd(password, **kwargs):
     '{SHA}IGyAQTualsExLMNGt9JRe4RGPt0='
     """
 
-    return "{SHA}%s" % base64.b64encode(sha1(password).digest())
+    password = getBytes(password)
+
+    return "{SHA}%s" % getText(base64.b64encode(sha1(password).digest()))
 
 def ssha_passwd(password, salt, **kwargs):
     """
@@ -275,7 +300,10 @@ def ssha_passwd(password, salt, **kwargs):
     '{SSHA}mU1HPTvnmoXOhE4ROHP6sWfbfoRzYWx0'
     """
 
-    return "{SSHA}%s" % base64.b64encode(sha1(password + salt).digest() + salt)
+    password = getBytes(password)
+    salt = getBytes(salt)
+
+    return "{SSHA}%s" % getText(base64.b64encode(sha1(password + salt).digest() + salt))
 
 def ssha256_passwd(password, salt, **kwargs):
     """
@@ -283,7 +311,10 @@ def ssha256_passwd(password, salt, **kwargs):
     '{SSHA256}hhubsLrO/Aje9F/kJrgv5ZLE40UmTrVWvI7Dt6InP99zYWx0'
     """
 
-    return "{SSHA256}%s" % base64.b64encode(sha256(password + salt).digest() + salt)
+    password = getBytes(password)
+    salt = getBytes(salt)
+
+    return "{SSHA256}%s" % getText(base64.b64encode(sha256(password + salt).digest() + salt))
 
 def ssha512_passwd(password, salt, **kwargs):
     """
@@ -291,7 +322,10 @@ def ssha512_passwd(password, salt, **kwargs):
     '{SSHA512}mCUSLfPMhXCQOJl9WHW/QMn9v9sjq7Ht/Wk7iVau8vLOfh+PeynkGMikqIE8sStFd0khdfcCD8xZmC6UyjTxsHNhbHQ='
     """
 
-    return "{SSHA512}%s" % base64.b64encode(sha512(password + salt).digest() + salt)
+    password = getBytes(password)
+    salt = getBytes(salt)
+
+    return "{SSHA512}%s" % getText(base64.b64encode(sha512(password + salt).digest() + salt))
 
 def sha224_generic_passwd(password, uppercase=False):
     """
@@ -299,7 +333,7 @@ def sha224_generic_passwd(password, uppercase=False):
     '648db6019764b598f75ab6b7616d2e82563a00eb1531680e19ac4c6f'
     """
 
-    retVal = sha224(password).hexdigest()
+    retVal = sha224(getBytes(password)).hexdigest()
 
     return retVal.upper() if uppercase else retVal.lower()
 
@@ -309,7 +343,7 @@ def sha256_generic_passwd(password, uppercase=False):
     '13d249f2cb4127b40cfa757866850278793f814ded3c587fe5889e889a7a9f6c'
     """
 
-    retVal = sha256(password).hexdigest()
+    retVal = sha256(getBytes(password)).hexdigest()
 
     return retVal.upper() if uppercase else retVal.lower()
 
@@ -319,7 +353,7 @@ def sha384_generic_passwd(password, uppercase=False):
     '6823546e56adf46849343be991d4b1be9b432e42ed1b4bb90635a0e4b930e49b9ca007bc3e04bf0a4e0df6f1f82769bf'
     """
 
-    retVal = sha384(password).hexdigest()
+    retVal = sha384(getBytes(password)).hexdigest()
 
     return retVal.upper() if uppercase else retVal.lower()
 
@@ -329,7 +363,7 @@ def sha512_generic_passwd(password, uppercase=False):
     '78ddc8555bb1677ff5af75ba5fc02cb30bb592b0610277ae15055e189b77fe3fda496e5027a3d99ec85d54941adee1cc174b50438fdc21d82d0a79f85b58cf44'
     """
 
-    retVal = sha512(password).hexdigest()
+    retVal = sha512(getBytes(password)).hexdigest()
 
     return retVal.upper() if uppercase else retVal.lower()
 
@@ -366,14 +400,9 @@ def unix_md5_passwd(password, salt, magic="$1$", **kwargs):
 
         return output
 
-    if isinstance(password, unicode):
-        password = password.encode(UNICODE_ENCODING)
-
-    if isinstance(magic, unicode):
-        magic = magic.encode(UNICODE_ENCODING)
-
-    if isinstance(salt, unicode):
-        salt = salt.encode(UNICODE_ENCODING)
+    password = getBytes(password)
+    magic = getBytes(magic)
+    salt = getBytes(salt)
 
     salt = salt[:8]
     ctx = password + magic + salt
@@ -388,15 +417,15 @@ def unix_md5_passwd(password, salt, magic="$1$", **kwargs):
     i = len(password)
     while i:
         if i & 1:
-            ctx = ctx + chr(0)  # if ($i & 1) { $ctx->add(pack("C", 0)); }
+            ctx = ctx + b'\x00'  # if ($i & 1) { $ctx->add(pack("C", 0)); }
         else:
-            ctx = ctx + password[0]
+            ctx = ctx + password[0:1]
         i = i >> 1
 
     final = md5(ctx).digest()
 
     for i in xrange(1000):
-        ctx1 = ""
+        ctx1 = b""
 
         if i & 1:
             ctx1 = ctx1 + password
@@ -416,14 +445,14 @@ def unix_md5_passwd(password, salt, magic="$1$", **kwargs):
 
         final = md5(ctx1).digest()
 
-    hash_ = _encode64((int(ord(final[0])) << 16) | (int(ord(final[6])) << 8) | (int(ord(final[12]))), 4)
-    hash_ = hash_ + _encode64((int(ord(final[1])) << 16) | (int(ord(final[7])) << 8) | (int(ord(final[13]))), 4)
-    hash_ = hash_ + _encode64((int(ord(final[2])) << 16) | (int(ord(final[8])) << 8) | (int(ord(final[14]))), 4)
-    hash_ = hash_ + _encode64((int(ord(final[3])) << 16) | (int(ord(final[9])) << 8) | (int(ord(final[15]))), 4)
-    hash_ = hash_ + _encode64((int(ord(final[4])) << 16) | (int(ord(final[10])) << 8) | (int(ord(final[5]))), 4)
-    hash_ = hash_ + _encode64((int(ord(final[11]))), 2)
+    hash_ = _encode64((int(ord(final[0:1])) << 16) | (int(ord(final[6:7])) << 8) | (int(ord(final[12:13]))), 4)
+    hash_ = hash_ + _encode64((int(ord(final[1:2])) << 16) | (int(ord(final[7:8])) << 8) | (int(ord(final[13:14]))), 4)
+    hash_ = hash_ + _encode64((int(ord(final[2:3])) << 16) | (int(ord(final[8:9])) << 8) | (int(ord(final[14:15]))), 4)
+    hash_ = hash_ + _encode64((int(ord(final[3:4])) << 16) | (int(ord(final[9:10])) << 8) | (int(ord(final[15:16]))), 4)
+    hash_ = hash_ + _encode64((int(ord(final[4:5])) << 16) | (int(ord(final[10:11])) << 8) | (int(ord(final[5:6]))), 4)
+    hash_ = hash_ + _encode64((int(ord(final[11:12]))), 2)
 
-    return "%s%s$%s" % (magic, salt, hash_)
+    return getText(magic + salt + b'$' + getBytes(hash_))
 
 def joomla_passwd(password, salt, **kwargs):
     """
@@ -433,7 +462,7 @@ def joomla_passwd(password, salt, **kwargs):
     'e3d5794da74e917637332e0d21b76328:6GGlnaquVXI80b3HRmSyE3K1wEFFaBIf'
     """
 
-    return "%s:%s" % (md5("%s%s" % (password, salt)).hexdigest(), salt)
+    return "%s:%s" % (md5(getBytes(password) + getBytes(salt)).hexdigest(), salt)
 
 def django_md5_passwd(password, salt, **kwargs):
     """
@@ -443,7 +472,7 @@ def django_md5_passwd(password, salt, **kwargs):
     'md5$salt$972141bcbcb6a0acc96e92309175b3c5'
     """
 
-    return "md5$%s$%s" % (salt, md5("%s%s" % (salt, password)).hexdigest())
+    return "md5$%s$%s" % (salt, md5(getBytes(salt) + getBytes(password)).hexdigest())
 
 def django_sha1_passwd(password, salt, **kwargs):
     """
@@ -453,7 +482,7 @@ def django_sha1_passwd(password, salt, **kwargs):
     'sha1$salt$6ce0e522aba69d8baa873f01420fccd0250fc5b2'
     """
 
-    return "sha1$%s$%s" % (salt, sha1("%s%s" % (salt, password)).hexdigest())
+    return "sha1$%s$%s" % (salt, sha1(getBytes(salt) + getBytes(password)).hexdigest())
 
 def vbulletin_passwd(password, salt, **kwargs):
     """
@@ -463,7 +492,7 @@ def vbulletin_passwd(password, salt, **kwargs):
     '85c4d8ea77ebef2236fb7e9d24ba9482:salt'
     """
 
-    return "%s:%s" % (md5("%s%s" % (md5(password).hexdigest(), salt)).hexdigest(), salt)
+    return "%s:%s" % (md5(binascii.hexlify(md5(getBytes(password)).digest()) + getBytes(salt)).hexdigest(), salt)
 
 def wordpress_passwd(password, salt, count, prefix, **kwargs):
     """
@@ -480,12 +509,12 @@ def wordpress_passwd(password, salt, count, prefix, **kwargs):
         i = 0
 
         while i < count:
-            value = ord(input_[i])
+            value = (input_[i] if isinstance(input_[i], int) else ord(input_[i]))
             i += 1
             output = output + ITOA64[value & 0x3f]
 
             if i < count:
-                value = value | (ord(input_[i]) << 8)
+                value = value | ((input_[i] if isinstance(input_[i], int) else ord(input_[i])) << 8)
 
             output = output + ITOA64[(value >> 6) & 0x3f]
 
@@ -494,7 +523,7 @@ def wordpress_passwd(password, salt, count, prefix, **kwargs):
                 break
 
             if i < count:
-                value = value | (ord(input_[i]) << 16)
+                value = value | ((input_[i] if isinstance(input_[i], int) else ord(input_[i])) << 16)
 
             output = output + ITOA64[(value >> 12) & 0x3f]
 
@@ -506,8 +535,8 @@ def wordpress_passwd(password, salt, count, prefix, **kwargs):
 
         return output
 
-    if isinstance(password, unicode):
-        password = password.encode(UNICODE_ENCODING)
+    password = getBytes(password)
+    salt = getBytes(salt)
 
     cipher = md5(salt)
     cipher.update(password)
@@ -558,7 +587,7 @@ def storeHashesToFile(attack_dict):
     if not attack_dict:
         return
 
-    items = oset()
+    items = OrderedSet()
 
     for user, hashes in attack_dict.items():
         for hash_ in hashes:
@@ -566,9 +595,9 @@ def storeHashesToFile(attack_dict):
             if hash_ and hash_ != NULL and hashRecognition(hash_):
                 item = None
                 if user and not user.startswith(DUMMY_USER_PREFIX):
-                    item = "%s:%s\n" % (user.encode(UNICODE_ENCODING), hash_.encode(UNICODE_ENCODING))
+                    item = "%s:%s\n" % (user, hash_)
                 else:
-                    item = "%s\n" % hash_.encode(UNICODE_ENCODING)
+                    item = "%s\n" % hash_
 
                 if item and item not in items:
                     items.add(item)
@@ -586,7 +615,7 @@ def storeHashesToFile(attack_dict):
         infoMsg = "writing hashes to a temporary file '%s' " % filename
         logger.info(infoMsg)
 
-        with open(filename, "w+") as f:
+        with openFile(filename, "w+") as f:
             for item in items:
                 f.write(item)
 
@@ -598,7 +627,7 @@ def attackCachedUsersPasswords():
         for (_, hash_, password) in results:
             lut[hash_.lower()] = password
 
-        for user in kb.data.cachedUsersPasswords.keys():
+        for user in kb.data.cachedUsersPasswords:
             for i in xrange(len(kb.data.cachedUsersPasswords[user])):
                 if (kb.data.cachedUsersPasswords[user][i] or "").strip():
                     value = kb.data.cachedUsersPasswords[user][i].lower().split()[0]
@@ -608,7 +637,7 @@ def attackCachedUsersPasswords():
 def attackDumpedTable():
     if kb.data.dumpedTable:
         table = kb.data.dumpedTable
-        columns = table.keys()
+        columns = list(table.keys())
         count = table["__infos__"]["count"]
 
         if not count:
@@ -621,12 +650,25 @@ def attackDumpedTable():
         col_user = ''
         col_passwords = set()
         attack_dict = {}
+        binary_fields = OrderedSet()
+        replacements = {}
 
-        for column in columns:
+        for column in sorted(columns, key=len, reverse=True):
             if column and column.lower() in COMMON_USER_COLUMNS:
                 col_user = column
                 break
 
+        for column in columns:
+            if column != "__infos__" and table[column]["values"]:
+                if all(INVALID_UNICODE_CHAR_FORMAT.split('%')[0] in (value or "") for value in table[column]["values"]):
+                    binary_fields.add(column)
+
+        if binary_fields:
+            _ = ','.join(binary_fields)
+            warnMsg = "potential binary fields detected ('%s'). In case of any problems you are " % _
+            warnMsg += "advised to rerun table dump with '--fresh-queries --binary-fields=\"%s\"'" % _
+            logger.warn(warnMsg)
+
         for i in xrange(count):
             if not found and i > HASH_RECOGNITION_QUIT_THRESHOLD:
                 break
@@ -640,6 +682,11 @@ def attackDumpedTable():
 
                 value = table[column]["values"][i]
 
+                if column in binary_fields and re.search(HASH_BINARY_COLUMNS_REGEX, column) is not None:
+                    previous = value
+                    value = encodeHex(getBytes(value), binary=False)
+                    replacements[value] = previous
+
                 if hashRecognition(value):
                     found = True
 
@@ -673,7 +720,9 @@ def attackDumpedTable():
 
             for (_, hash_, password) in results:
                 if hash_:
-                    lut[hash_.lower()] = password
+                    key = hash_ if hash_ not in replacements else replacements[hash_]
+                    lut[key.lower()] = password
+                    lut["0x%s" % key.lower()] = password
 
             debugMsg = "post-processing table dump"
             logger.debug(debugMsg)
@@ -684,7 +733,7 @@ def attackDumpedTable():
                         value = table[column]['values'][i]
 
                         if value and value.lower() in lut:
-                            table[column]['values'][i] = "%s (%s)" % (getUnicode(table[column]['values'][i]), getUnicode(lut[value.lower()]))
+                            table[column]['values'][i] = "%s (%s)" % (getUnicode(table[column]['values'][i]), getUnicode(lut[value.lower()] or HASH_EMPTY_PASSWORD_MARKER))
                             table[column]['length'] = max(table[column]['length'], len(table[column]['values'][i]))
 
 def hashRecognition(value):
@@ -692,7 +741,7 @@ def hashRecognition(value):
 
     isOracle, isMySQL = Backend.isDbms(DBMS.ORACLE), Backend.isDbms(DBMS.MYSQL)
 
-    if isinstance(value, basestring):
+    if isinstance(value, six.string_types):
         for name, regex in getPublicTypeMembers(HASH):
             # Hashes for Oracle and old MySQL look the same hence these checks
             if isOracle and regex == HASH.MYSQL_OLD or isMySQL and regex == HASH.ORACLE_OLD:
@@ -723,7 +772,9 @@ def _bruteProcessVariantA(attack_info, hash_regex, suffix, retVal, proc_id, proc
 
             count += 1
 
-            if not isinstance(word, basestring):
+            if isinstance(word, six.binary_type):
+                word = getUnicode(word)
+            elif not isinstance(word, six.string_types):
                 continue
 
             if suffix:
@@ -769,8 +820,8 @@ def _bruteProcessVariantA(attack_info, hash_regex, suffix, retVal, proc_id, proc
             except (UnicodeEncodeError, UnicodeDecodeError):
                 pass  # ignore possible encoding problems caused by some words in custom dictionaries
 
-            except Exception, e:
-                warnMsg = "there was a problem while hashing entry: %s (%s). " % (repr(word), e)
+            except Exception as ex:
+                warnMsg = "there was a problem while hashing entry: %s ('%s'). " % (repr(word), getSafeExString(ex))
                 warnMsg += "Please report by e-mail to '%s'" % DEV_EMAIL_ADDRESS
                 logger.critical(warnMsg)
 
@@ -798,7 +849,9 @@ def _bruteProcessVariantB(user, hash_, kwargs, hash_regex, suffix, retVal, found
 
             count += 1
 
-            if not isinstance(word, basestring):
+            if isinstance(word, six.binary_type):
+                word = getUnicode(word)
+            elif not isinstance(word, six.string_types):
                 continue
 
             if suffix:
@@ -846,8 +899,8 @@ def _bruteProcessVariantB(user, hash_, kwargs, hash_regex, suffix, retVal, found
             except (UnicodeEncodeError, UnicodeDecodeError):
                 pass  # ignore possible encoding problems caused by some words in custom dictionaries
 
-            except Exception, e:
-                warnMsg = "there was a problem while hashing entry: %s (%s). " % (repr(word), e)
+            except Exception as ex:
+                warnMsg = "there was a problem while hashing entry: %s ('%s'). " % (repr(word), getSafeExString(ex))
                 warnMsg += "Please report by e-mail to '%s'" % DEV_EMAIL_ADDRESS
                 logger.critical(warnMsg)
 
@@ -879,7 +932,7 @@ def dictionaryAttack(attack_dict):
 
             if regex and regex not in hash_regexes:
                 hash_regexes.append(regex)
-                infoMsg = "using hash method '%s'" % __functions__[regex].func_name
+                infoMsg = "using hash method '%s'" % __functions__[regex].__name__
                 logger.info(infoMsg)
 
     for hash_regex in hash_regexes:
@@ -902,15 +955,17 @@ def dictionaryAttack(attack_dict):
                             hash_ = hash_.lower()
 
                         if hash_regex in (HASH.MD5_BASE64, HASH.SHA1_BASE64, HASH.SHA256_BASE64, HASH.SHA512_BASE64):
-                            item = [(user, hash_.decode("base64").encode("hex")), {}]
+                            item = [(user, encodeHex(decodeBase64(hash_, binary=True))), {}]
                         elif hash_regex in (HASH.MYSQL, HASH.MYSQL_OLD, HASH.MD5_GENERIC, HASH.SHA1_GENERIC, HASH.SHA224_GENERIC, HASH.SHA256_GENERIC, HASH.SHA384_GENERIC, HASH.SHA512_GENERIC, HASH.APACHE_SHA1):
+                            if hash_.startswith("0x"):  # Reference: https://docs.microsoft.com/en-us/sql/t-sql/functions/hashbytes-transact-sql?view=sql-server-2017
+                                hash_ = hash_[2:]
                             item = [(user, hash_), {}]
                         elif hash_regex in (HASH.SSHA,):
-                            item = [(user, hash_), {"salt": hash_.decode("base64")[20:]}]
+                            item = [(user, hash_), {"salt": decodeBase64(hash_, binary=True)[20:]}]
                         elif hash_regex in (HASH.SSHA256,):
-                            item = [(user, hash_), {"salt": hash_.decode("base64")[32:]}]
+                            item = [(user, hash_), {"salt": decodeBase64(hash_, binary=True)[32:]}]
                         elif hash_regex in (HASH.SSHA512,):
-                            item = [(user, hash_), {"salt": hash_.decode("base64")[64:]}]
+                            item = [(user, hash_), {"salt": decodeBase64(hash_, binary=True)[64:]}]
                         elif hash_regex in (HASH.ORACLE_OLD, HASH.POSTGRES):
                             item = [(user, hash_), {'username': user}]
                         elif hash_regex in (HASH.ORACLE,):
@@ -982,12 +1037,12 @@ def dictionaryAttack(attack_dict):
                     else:
                         logger.info("using default dictionary")
 
-                    dictPaths = filter(None, dictPaths)
+                    dictPaths = [_ for _ in dictPaths if _]
 
                     for dictPath in dictPaths:
                         checkFile(dictPath)
 
-                        if os.path.splitext(dictPath)[1].lower() == ".zip":
+                        if isZipFile(dictPath):
                             _ = zipfile.ZipFile(dictPath, 'r')
                             if len(_.namelist()) == 0:
                                 errMsg = "no file(s) inside '%s'" % dictPath
@@ -997,7 +1052,7 @@ def dictionaryAttack(attack_dict):
 
                     kb.wordlists = dictPaths
 
-                except Exception, ex:
+                except Exception as ex:
                     warnMsg = "there was a problem while loading dictionaries"
                     warnMsg += " ('%s')" % getSafeExString(ex)
                     logger.critical(warnMsg)
@@ -1007,7 +1062,7 @@ def dictionaryAttack(attack_dict):
             if readInput(message, default='N', boolean=True):
                 suffix_list += COMMON_PASSWORD_SUFFIXES
 
-        infoMsg = "starting dictionary-based cracking (%s)" % __functions__[hash_regex].func_name
+        infoMsg = "starting dictionary-based cracking (%s)" % __functions__[hash_regex].__name__
         logger.info(infoMsg)
 
         for item in attack_info:
@@ -1056,11 +1111,11 @@ def dictionaryAttack(attack_dict):
                         warnMsg += "not supported on this platform"
                         singleTimeWarnMessage(warnMsg)
 
-                        retVal = Queue()
+                        retVal = _queue.Queue()
                         _bruteProcessVariantA(attack_info, hash_regex, suffix, retVal, 0, 1, kb.wordlists, custom_wordlist, conf.api)
 
                 except KeyboardInterrupt:
-                    print
+                    print()
                     processException = True
                     warnMsg = "user aborted during dictionary-based attack phase (Ctrl+C was pressed)"
                     logger.warn(warnMsg)
@@ -1077,15 +1132,17 @@ def dictionaryAttack(attack_dict):
                         gc.enable()
 
                     if retVal:
-                        conf.hashDB.beginTransaction()
+                        if conf.hashDB:
+                            conf.hashDB.beginTransaction()
 
                         while not retVal.empty():
                             user, hash_, word = item = retVal.get(block=False)
-                            attack_info = filter(lambda _: _[0][0] != user or _[0][1] != hash_, attack_info)
+                            attack_info = [_ for _ in attack_info if _[0][0] != user or _[0][1] != hash_]
                             hashDBWrite(hash_, word)
                             results.append(item)
 
-                        conf.hashDB.endTransaction()
+                        if conf.hashDB:
+                            conf.hashDB.endTransaction()
 
             clearConsoleLine()
 
@@ -1142,10 +1199,10 @@ def dictionaryAttack(attack_dict):
                             warnMsg += "not supported on this platform"
                             singleTimeWarnMessage(warnMsg)
 
-                            class Value():
+                            class Value(object):
                                 pass
 
-                            retVal = Queue()
+                            retVal = _queue.Queue()
                             found_ = Value()
                             found_.value = False
 
@@ -1154,7 +1211,7 @@ def dictionaryAttack(attack_dict):
                             found = found_.value
 
                     except KeyboardInterrupt:
-                        print
+                        print()
                         processException = True
                         warnMsg = "user aborted during dictionary-based attack phase (Ctrl+C was pressed)"
                         logger.warn(warnMsg)
@@ -1170,15 +1227,17 @@ def dictionaryAttack(attack_dict):
                         if _multiprocessing:
                             gc.enable()
 
-                        if retVal:
-                            conf.hashDB.beginTransaction()
+                        if retVal and conf.hashDB:
+                            if conf.hashDB:
+                                conf.hashDB.beginTransaction()
 
                             while not retVal.empty():
                                 user, hash_, word = item = retVal.get(block=False)
                                 hashDBWrite(hash_, word)
                                 results.append(item)
 
-                            conf.hashDB.endTransaction()
+                            if conf.hashDB:
+                                conf.hashDB.endTransaction()
 
                 clearConsoleLine()
 
@@ -1193,3 +1252,17 @@ def dictionaryAttack(attack_dict):
         logger.warn(warnMsg)
 
     return results
+
+def crackHashFile(hashFile):
+    i = 0
+    attack_dict = {}
+
+    for line in getFileItems(conf.hashFile):
+        if ':' in line:
+            user, hash_ = line.split(':', 1)
+            attack_dict[user] = [hash_]
+        else:
+            attack_dict["%s%d" % (DUMMY_USER_PREFIX, i)] = [line]
+            i += 1
+
+    dictionaryAttack(attack_dict)
